@@ -1228,12 +1228,14 @@ notif_instantiate_surface(struct notif *notif, int *width, int *height)
     *height = (int)roundf(roundf(*height / scale) * scale);
 
     enum zwlr_layer_surface_v1_anchor anchor
-        = (conf->anchor == ANCHOR_TOP_LEFT || conf->anchor == ANCHOR_TOP_RIGHT
-           ? ZWLR_LAYER_SURFACE_V1_ANCHOR_TOP
-           : ZWLR_LAYER_SURFACE_V1_ANCHOR_BOTTOM)
-        | (conf->anchor == ANCHOR_TOP_LEFT || conf->anchor == ANCHOR_BOTTOM_LEFT
-           ? ZWLR_LAYER_SURFACE_V1_ANCHOR_LEFT
-           : ZWLR_LAYER_SURFACE_V1_ANCHOR_RIGHT);
+        = conf->anchor == ANCHOR_CENTER
+            ? 0
+            : (conf->anchor == ANCHOR_TOP_LEFT || conf->anchor == ANCHOR_TOP_RIGHT
+                ? ZWLR_LAYER_SURFACE_V1_ANCHOR_TOP
+                : ZWLR_LAYER_SURFACE_V1_ANCHOR_BOTTOM) |
+              (conf->anchor == ANCHOR_TOP_LEFT || conf->anchor == ANCHOR_BOTTOM_LEFT
+                  ? ZWLR_LAYER_SURFACE_V1_ANCHOR_LEFT
+                  : ZWLR_LAYER_SURFACE_V1_ANCHOR_RIGHT);
 
     zwlr_layer_surface_v1_set_anchor(layer_surface, anchor);
     zwlr_layer_surface_v1_set_size(
